import {Component, HostListener, OnDestroy, OnInit} from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {TabuMiddlewareService} from '../dao/TabuMiddlewareService';
import {GameStatus} from '../interface/gameStatus';
import {IsAllowedToPlay} from '../dao/isAllowedToPlay';
import {SocketDataService} from '../dao/socketDataService';
import {Subject} from 'rxjs';
import {takeUntil} from 'rxjs/operators';
import {ViewModus} from '../dao/viewModus';
import {Location} from '@angular/common';
import {MatDialog} from '@angular/material/dialog';
import {HttpErrorResponse} from '@angular/common/http';

@Component({
  selector: 'app-overview',
  templateUrl: './overview.component.html',
  styleUrls: ['./overview.component.scss']
})
export class OverviewComponent implements OnInit, OnDestroy {

  constructor(private router: Router,
              private location: Location,
              private activatedRoute: ActivatedRoute,
              private service: TabuMiddlewareService,
              private isAllowedToPlay: IsAllowedToPlay,
              private socketDataService: SocketDataService,
              private screen: ViewModus,
              public dialog: MatDialog) {
    screen.$view.subscribe(
      view => {
        this.view = view;
        this.setScrollLimit();
      });
  }
  view = 'desktopView';
  private ngUnsubscribe = new Subject();
  subNewGame: any;
  subNewCard: any;
  subNewRound: any;
  subEndRound: any;
  subHistoryChanged: any;
  subSessionPlayers: any;
  wantToBeExplainer = false;
  isActiveRound = false;
  red = 'choose';
  blue = 'choose';
  disablePlayersOnlineBadges = true;
  totalPlayers = 1;
  redPlayers = 0;
  bluePlayers = 0;
  nextTeam = 'Blau';
  team = '';
  teamNumber = 0;
  buttonNewGameDisabled = true;
  buttonNextRoundDisabled = true;
  sessionName = '';
  membership = 'Wähle dein Team';
  gameStatus: GameStatus = {
    sessionID: -1,
    round: 0,
    red: 0,
    blue: 0,
    redTurn: true,
    activeExplainer: 1,
    activeCard: 1,
    isTabooh: 1,
    lastPlay: '01/01/9999'
  };
  history: any = [];
  canUpdateHistory = true;
  scrolled = 0;
  scrollLimit = 290;

  static playAudio(audioPath: string): void {
    const audio = new Audio();
    audio.src = audioPath;
    audio.load();
    audio.play();
  }

  ngOnInit(): void {
    this.view = this.screen.actuallView;
    this.setScrollLimit();
    this.activatedRoute.paramMap.subscribe(params => {
      this.sessionName = String(params.get('sessionName')).toLowerCase();
      this.team = String(params.get('team'));
      this.teamToTeamNumber(this.team);
      this.getSocketData();
      this.service.isSession({spielname: this.sessionName}).then(value => {
        if (!JSON.parse(value.status)) {
          this.router.navigate(['error']);
        } else {
          this.selectTeam();
          this.getGameStatus();
        }
      });
    });
  }

  ngOnDestroy(): void {
    this.unsubscribeAll();
  }

  getSocketData(): void {
    this.subHistoryChanged = this.socketDataService.getHistoryChangedInfo (this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe)).subscribe( data => {
        if (this.canUpdateHistory) {
          this.getGameStatus();
        }
      });
    this.subNewGame = this.socketDataService.getNewGame (this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe)).subscribe(data => {
        this.getGameStatus();
      });
    this.subNewCard = this.socketDataService.getNewCard(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe)).subscribe(data => {
        this.getGameStatus();
      });
    this.subEndRound = this.socketDataService.getEndRound(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe)).subscribe(data => {
        if (JSON.parse(data)) {
          this.getGameStatus();
        }
      });
    this.subNewRound = this.socketDataService.getNewRound(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe)).subscribe(data => {
        if (JSON.parse(data)) {
          if (((this.team === 'red' && this.nextTeam === 'Rot') || (this.team === 'blue' && this.nextTeam === 'Blau'))
            && !this.wantToBeExplainer) {
            this.isAllowedToPlay.isAllowed = true;
            this.isAllowedToPlay.role = 'guesser';
            this.unsubscribeAll();
            this.router.navigate([this.sessionName + '/' + this.team + '/guesser']);
          } else if ((this.team === 'red' || this.team === 'blue') && !this.wantToBeExplainer) {
            this.isAllowedToPlay.isAllowed = true;
            this.isAllowedToPlay.role = 'watchdog';
            this.unsubscribeAll();
            this.router.navigate([this.sessionName + '/' + this.team + '/watchdog']);
          } else {
            this.getGameStatus();
          }
        }
      });
    this.subSessionPlayers = this.socketDataService.getSessionPlayers(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe)).subscribe(data => {
        if (data.hasOwnProperty('total')) {
          this.disablePlayersOnlineBadges = false;
          this.totalPlayers = data.total;
          this.redPlayers = data.red;
          this.bluePlayers = data.blue;
        }
      });
    this.socketDataService.updateSessionPlayers(this.sessionName, this.teamNumber, 1);
  }

  selectTeam(): void {
    if (this.team === 'red'){
      this.membership = 'Du gehörst zu Team rot!';
      this.buttonNewGameDisabled = false;
      this.red = 'red';
      this.blue = 'choose';
    }
    else if (this.team === 'blue'){
      this.membership = 'Du gehörst zu Team blau!';
      this.buttonNewGameDisabled = false;
      this.blue = 'blue';
      this.red = 'choose';
    }
    else if (this.team === 'null'){
      this.membership = 'Wähle dein Team';
    }
    else {
      this.router.navigate(['error']);
      return;
    }
  }

  canStartGame(): void {
    if (this.gameStatus.redTurn){
      this.nextTeam = 'Rot';
      this.buttonNextRoundDisabled = this.team !== 'red';
    }
    else{
      this.nextTeam = 'Blau';
      this.buttonNextRoundDisabled = this.team !== 'blue';
    }
  }

  startGame(): void {
    if (this.team === 'red' || this.team === 'blue') {
      this.service.newGame({spielname: this.sessionName}).then(value => {
        const status = JSON.parse(value.status);
        if (status) {
          this.getGameStatus();
          const audio = '../../../assets/sounds/ui_refresh-feed.ogg';
          OverviewComponent.playAudio(audio);
        }
      }).catch(reason => {
        console.log(reason);
      });
    }
  }

  nextRound(): void {
    this.unsubscribeAll();
    this.service.newRound({spielname: this.sessionName}).then( valueNotUsed => {
      this.service.getS2C({spielname: this.sessionName}).then(value => {
        this.isAllowedToPlay.isAllowed = true;
        this.isAllowedToPlay.role = 'explainer';
        this.router.navigate([this.sessionName + '/' + this.team + '/explainer', ]);
        this.wantToBeExplainer = true;
        const audio = "../../../assets/sounds/state-change_confirm-up.ogg";
        OverviewComponent.playAudio(audio);
      });
    }).catch((err: HttpErrorResponse) => {
      if (err.status === 502) {
        this.openDialog();
      }
    });
  }

  joinTeam(team: string): void {
    this.team = team;
    this.teamToTeamNumber(this.team);
    this.unsubscribeAll();
    this.location.replaceState(this.sessionName + '/' + team);
    this.selectTeam();
    this.canStartGame();
    this.unsubscribeAll();
    this.getSocketData();
    const audio = "../../../assets/sounds/ui_unlock.ogg";
    OverviewComponent.playAudio(audio);
  }

  teamToTeamNumber(team: string): void {
    if (team === 'red') {
      this.teamNumber = 1;
    }
    if (team === 'blue') {
      this.teamNumber = 2;
    }
    if (team === '') {
      this.teamNumber = 0;
    }
  }

  getGameStatus(): void {
    this.service.getGamestatus({spielname: this.sessionName}).then(value => {
      this.fillGameStatus(value);
      this.getHistory();
    }).catch(reason => console.log(reason));
  }

  fillGameStatus(value: any): void {
    const status = JSON.parse(value.status);
    if (status) {
      this.gameStatus.sessionID = JSON.parse(value.sessionID);
      this.gameStatus.round = JSON.parse(value.round);
      this.gameStatus.red = JSON.parse(value.red);
      this.gameStatus.blue = JSON.parse(value.blue);
      this.gameStatus.redTurn = JSON.parse(value.redTurn);
      this.gameStatus.activeExplainer = JSON.parse(value.activeExplainer);
      this.gameStatus.activeCard = JSON.parse(value.activeCard);
      this.isActiveRound = this.gameStatus.activeExplainer === 1;
      this.canStartGame();
    }
  }

  updateChangeHistory(value: boolean): void {
    this.canUpdateHistory = value;
  }

  getArrayRounds(): any[] {
    return Array(this.gameStatus.round);
  }

  getHistory(): void {
    this.service.getSessionHistory({spielname: this.sessionName}).then(value => {
      const status = JSON.parse(value.status);
      if (status) {
        this.history = value.history;
      }
    });
  }

  setScrollLimit(): void {
    if (this.view === 'mobileView') {
      this.scrollLimit = 325;
    } else {
      this.scrollLimit = 290;
    }
  }

  openDialog(): void {
    this.dialog.open(TestDialogComponent);
  }

  @HostListener('window:scroll', ['$event'])
  onWindowScroll($event: any): void {
    const numb = window.scrollY;
    if (numb >= this.scrollLimit){
      this.scrolled = 1;
    }
    else {
      this.scrolled = 0;
    }
  }

  unsubscribeAll(): void {
    this.subNewCard.unsubscribe();
    this.subNewRound.unsubscribe();
    this.subEndRound.unsubscribe();
    this.subNewGame.unsubscribe();
    this.subHistoryChanged.unsubscribe();
    this.subSessionPlayers.unsubscribe();
    this.ngUnsubscribe.next();
    this.ngUnsubscribe.complete();
  }
}
@Component({
  selector: 'app-testdialog',
  templateUrl: './testdialog.html',
})
export class TestDialogComponent {}
